import { JSONObject, JSONObjectOf } from '../../json';
import { JSONLikeArrayOf, JSONLikeObject } from '../../jsonLike';
import { ServiceClient } from '../JSONRPC11/ServiceClient';
import { EpochTimeMS, Mapping, SDKBoolean } from '../types';
export interface ObjectIdentity extends JSONLikeObject {
    workspace?: string;
    wsid?: number;
    name?: string;
    objid?: string;
    ver?: number;
    ref?: string;
}
export interface WorkspaceIdentity extends JSONLikeObject {
    workspace?: string;
    id?: number;
}
export interface ObjectSpecification extends JSONLikeObject {
    workspace?: string;
    wsid?: number;
    name?: string;
    objid?: number;
    ver?: number;
    ref?: string;
    obj_path?: Array<ObjectIdentity>;
    obj_ref_path?: Array<string>;
    to_obj_path?: Array<ObjectIdentity>;
    to_obj_ref_path?: Array<string>;
    find_reference_path?: number;
    included?: string;
    strict_maps?: number;
    strict_arrays?: number;
}
export interface GetObjectInfo3Params {
    objects: Array<ObjectSpecification>;
    includeMetadata?: number;
    ignoreErrors?: number;
}
export type ObjectInfoRaw = [
    number,
    string,
    string,
    string,
    number,
    string,
    number,
    string,
    string,
    number,
    Metadata
];
export interface ObjectInfo {
    id: number;
    name: string;
    type: string;
    save_date: string;
    version: number;
    saved_by: string;
    wsid: number;
    ws: string;
    checksum: string;
    size: number;
    metadata: Metadata;
    ref: string;
    obj_id: string;
    typeModule: string;
    typeName: string;
    typeMajorVersion: string;
    typeMinorVersion: string;
    saveDate: string;
    savedAt: number;
}
export type WorkspaceInfoRaw = [
    number,
    string,
    string,
    string,
    number,
    string,
    string,
    string,
    Metadata
];
export interface WorkspaceInfo {
    id: number;
    name: string;
    owner: string;
    moddate: string;
    object_count: number;
    user_permission: string;
    globalread: string;
    lockstat: string;
    metadata: Metadata;
    modDate: string;
    modifiedAt: number;
}
export type Metadata = JSONObjectOf<string>;
export interface GetObjectInfo3Result extends JSONObject {
    infos: Array<ObjectInfoRaw>;
    paths: Array<Array<string>>;
}
export interface GetWorkspaceInfoParams extends WorkspaceIdentity {
}
export interface GetWorkspaceInfoResult {
}
export interface ListWorkspaceInfoParams {
    perm?: string;
    owners?: Array<string>;
    meta?: Metadata;
    after?: Timestamp;
    before?: Timestamp;
    after_epoch?: EpochTimeMS;
    before_epoch?: EpochTimeMS;
    excludeGlobal?: SDKBoolean;
    showDeleted?: SDKBoolean;
    showOnlyDeleted?: SDKBoolean;
}
export type ListWorkspaceInfoResult = Array<WorkspaceInfo>;
export type ObjectRef = string;
export type Username = string;
export type Timestamp = string;
export type WorkspaceID = number;
export type IDType = string;
export type RefString = string;
export interface ExternalDataUnit extends JSONLikeObject {
    resource_name: string;
    resource_url: string;
    resource_version: string;
    resource_release_date: Timestamp;
    resource_release_epoch: EpochTimeMS;
    data_url: string;
    data_id: string;
    description: string;
}
export interface SubAction extends JSONLikeObject {
    name: string;
    ver: string;
    code_url: string;
    commit: string;
    endpoint_url: string;
}
export interface ProvenanceAction extends JSONLikeObject {
    time: Timestamp;
    epoch: EpochTimeMS;
    caller: string;
    service: string;
    service_ver: string;
    method: string;
    method_params: Array<JSONObject>;
    script: string;
    script_ver: string;
    script_command_line: string;
    input_ws_objects: Array<RefString>;
    resolved_ws_objects: Array<ObjectRef>;
    intermediate_incoming: Array<string>;
    intermediate_outgoing: Array<string>;
    external_data: Array<ExternalDataUnit>;
    subactions: Array<SubAction>;
    custom: Mapping<string, string>;
    description: string;
}
export interface ObjectData extends JSONLikeObject {
    data: JSONObject;
    info: ObjectInfoRaw;
    path: Array<ObjectRef>;
    provenance: Array<ProvenanceAction>;
    creator: Username;
    orig_wsid: number;
    created: Timestamp;
    epoch: EpochTimeMS;
    refs: Array<ObjectRef>;
    copied: ObjectRef;
    copy_source_inaccessible: SDKBoolean;
    extracted_ids: Mapping<IDType, Array<string>>;
    handle_error: string;
    handle_stacktrace: string;
}
export declare function workspaceInfoToObject(wsInfo: WorkspaceInfoRaw): WorkspaceInfo;
export declare function objectInfoToObject(objInfo: ObjectInfoRaw): ObjectInfo;
export interface GetObjects2Param extends JSONLikeObject {
    objects: JSONLikeArrayOf<ObjectSpecification>;
    ignoreErrors?: SDKBoolean;
    no_data?: SDKBoolean;
}
export interface GetObjects2Result extends JSONLikeObject {
    data: Array<ObjectData>;
}
export interface GetPermissionsMassParams extends JSONLikeObject {
    workspaces: Array<WorkspaceIdentity>;
}
export type Perm = 'a' | 'w' | 'r' | 'n';
export interface UserPerm {
    [x: string]: Perm;
}
export interface GetPermissionsMassResult extends JSONLikeObject {
    perms: Array<UserPerm>;
}
export interface AlterWorkspaceMetadataParams extends JSONLikeObject {
    wsi: WorkspaceIdentity;
    new?: Metadata;
    remove?: Array<string>;
}
export type VerResult = string;
export interface StatusDependency extends JSONLikeObject {
    name: string;
    state: string;
    message: string;
    version: string;
}
export interface StatusResult extends JSONLikeObject {
    state: string;
    message: string;
    dependencies: Array<StatusDependency>;
    version: string;
    git_url: string;
    freemem: number;
    totalmem: number;
    maxmem: number;
}
export default class WorkspaceClient extends ServiceClient {
    module: string;
    get_object_info3(params: GetObjectInfo3Params): Promise<{
        infos: ObjectInfo[];
        paths: string[][];
    }>;
    get_workspace_info(params: GetWorkspaceInfoParams): Promise<WorkspaceInfo>;
    list_workspace_info(params: ListWorkspaceInfoParams): Promise<ListWorkspaceInfoResult>;
    get_objects2(params: GetObjects2Param): Promise<GetObjects2Result>;
    get_permissions_mass(params: GetPermissionsMassParams): Promise<GetPermissionsMassResult>;
    alter_workspace_metadata(params: AlterWorkspaceMetadataParams): Promise<void>;
    ver(): Promise<VerResult>;
    status(): Promise<StatusResult>;
}
